/* 
 * ==========================================================
 * Artekit demo sketch for the AK-DS1307 RTC breakout board
 * Copyright (c) 2013 Artekit Italy
 * http://www.artekit.eu
 *
 * Connect the pins in the following way:
 *
 * Arduino UNO          AK-DS1307
 * ==============================
 *   AN5                   SCL
 *   AN4                   SDA
 *   5V                    +5V
 *   GND                   GND
 *
 * This demo program shows a menu where you can configure
 * the AK-DS1307. Press any key to enter the menu.
 *
 * ==========================================================
 */
 
#include <Wire.h>

/* The DS1307 I2C Address */
#define DS1307_ADDR 0x68

/* We use these variables to read/write the current date */
static byte second, minute, hour, day, month, year, day_of_week;

/* Set this to one to access the menu mode */
static byte menu_mode = 0;

/* Strings for the days of the week */
static char* days_of_week[] =
{
  "Monday",
  "Tuesday",
  "Wednesday",
  "Thursday",
  "Friday",
  "Saturday",
  "Sunday"
};

/* Converts a value from decimal to BCD */
static byte DECtoBCD(byte val)
{
  return ( (val/10*16) + (val%10) );
}

/* Converts a value from BCD to decimal */
static byte BCDtoDEC(byte val)
{
  return ( (val/16*10) + (val%16) );
}

/* Reads the current time from the DS1307 */
static void GetTime()
{
  Wire.beginTransmission(DS1307_ADDR);
  Wire.write(byte(0x00));
  Wire.endTransmission();
  
  Wire.requestFrom(DS1307_ADDR, 7);

  second       = BCDtoDEC(Wire.read() & 0x7f);
  minute       = BCDtoDEC(Wire.read());
  hour         = BCDtoDEC(Wire.read() & 0x3f);  
  day_of_week  = BCDtoDEC(Wire.read());
  day          = BCDtoDEC(Wire.read());
  month        = BCDtoDEC(Wire.read());
  year         = BCDtoDEC(Wire.read());
}

/* Reads and show the current time */
static void ShowCurrentTime()
{
  GetTime();
  Serial.print("The current time is: ");
  
  /* Check if day_of_week has a good value */
  if (day_of_week > 0 && day_of_week < 7)
  {
    /* If so, print the string for the current day of the week */
    Serial.print(days_of_week[day_of_week-1]);
    Serial.print(" ");
  }
  
  Serial.print(month, DEC);
  Serial.print("/");
  Serial.print(day, DEC);
  Serial.print("/");
  Serial.print(year, DEC);
  Serial.print("  ");
  Serial.print(hour, DEC);
  Serial.print(":");
  Serial.print(minute, DEC);
  Serial.print(":");
  Serial.println(second, DEC);
}

/* This function takes the user input from the serial and sets the time
 * to the DS1307
 */
static bool ReceiveTime()
{
  int i;
  int val;
  
  Serial.println("Write the time in the following format: ");
  Serial.println("DMMDDYYHHMISS");
  Serial.println("D= Day of the week (1 to 7)");
  Serial.println("MM= Month");
  Serial.println("DD= Day");
  Serial.println("YY= Year");
  Serial.println("HH= Hour");
  Serial.println("MI= Minutes");
  Serial.println("SS= Seconds");
  Serial.println("For example: Monday 06/10/2013 19:01:22 will be 1061013190122");
  Serial.println("Press x to abort.");
  
  /* We know we have to read 13 values */
  for (i = 0; i < 13; i++)
  {
    while (!Serial.available());
    val = Serial.read();
    Serial.write(val);
    if (val == 'x' || val == 'X')
      return false;
 
    switch (i)
    {
      case 0:
        day_of_week = (byte) (val - 48);
        break;
       
      case 1:
        month = (byte) (val - 48) *10;
        break;
        
      case 2:
        month += val - 48;
        break;
        
      case 3:
        day = (byte) (val - 48) *10;
        break;
        
      case 4:
        day += (byte) val - 48;
        break;
        
      case 5:
        year = (byte) (val - 48) *10;
        break;
        
      case 6:
        year += (byte) val - 48;
        break;
       
      case 7:
        hour = (byte) (val - 48) *10;
        break;
        
      case 8:
        hour += (byte) val - 48;
        break;
        
      case 9:
        minute = (byte) (val - 48) * 10;
        break;
        
      case 10:
        minute += (byte) val - 48;
        break;
      
      case 11:
        second = (byte) (val - 48) * 10;
        break;
        
      case 12:
        second += (byte) val - 48;
        break;
    }
  }
    
  return true;
}

/* This function actually writes the time to the DS1307 */
static void WriteTime()
{
  Wire.beginTransmission(DS1307_ADDR);
  Wire.write(byte(0x00));
  Wire.write(DECtoBCD(second));
  Wire.write(DECtoBCD(minute));
  Wire.write(DECtoBCD(hour));
  Wire.write(DECtoBCD(day_of_week));
  Wire.write(DECtoBCD(day));
  Wire.write(DECtoBCD(month));
  Wire.write(DECtoBCD(year));
  Wire.endTransmission();
}

/* Ask the user the time and writes it to the DS1307 */
static void SetTime()
{
  if (ReceiveTime())
  {
    WriteTime();
    Serial.println("");
    ShowCurrentTime();
  }
}

/* Memory dump (64 bytes) */
static void DumpMemory()
{
  int i;
  byte val;
  
  Wire.beginTransmission(DS1307_ADDR);
  Wire.write(byte(0x00));
  Wire.endTransmission();
  Wire.requestFrom(DS1307_ADDR, 64);

  for (i = 1; i <= 64; i++)
  {
    val = Wire.read();
    
    if (i % 16 == 0)
    {
      Serial.println(val, HEX);
    } else {
      Serial.print(val, HEX);
      Serial.print(" ");
    }    
  }
}

/* Erases the memory beyond the date values */
static void EraseMemory()
{
  Serial.print("Erasing");
  Wire.beginTransmission(DS1307_ADDR);
          
  /* Say the DS1307 to jump to the position 0x08, right after
   * the date values.
   */
  Wire.write(byte(0x08));  
  for (int i = 0; i < 56; i++)
  {
    Wire.write(byte(0xff));
    delay(100);
    Serial.print(".");
  }   
  Wire.endTransmission();
  Serial.println("");
  Serial.println("Memory erased");
}

/* Shows the menu */
static void ShowMenu()
{
 Serial.println("");
 Serial.println("Artekit AK-DS1307");
 Serial.println("-----------------");
 Serial.println("");
 Serial.println("Choose one of the following options:");
 Serial.println("S - Set time");
 Serial.println("R - Read time");
 Serial.println("D - Dump memory contents");
 Serial.println("E - Erase memory");
 Serial.println("X - Exit");
 Serial.println("");
}

void setup()
{
  Wire.begin();
  Serial.begin(9600);
}

void loop()
{
  /* We use the menu mode to configure the DS1307. If not in menu mode,
   * output the date every second
   */
  if (menu_mode)
  {
    switch (Serial.read())
    {
      case 's':
      case 'S':
        SetTime();
        ShowMenu();
        break;
        
      case 'r':
      case 'R':
        ShowCurrentTime();
        ShowMenu();
        break;
        
      case 'd':
      case 'D':
        DumpMemory();
        ShowMenu();
        break;
        
      case 'e':
      case 'E':
        EraseMemory();
        ShowMenu();
        break;
        
      case 'x':
      case 'X':
        menu_mode = 0;
        break;
        
      case -1:
        break;
        
      default:
        Serial.println("Invalid option.");
        ShowMenu();
        break;
    }
  } else {
    if (Serial.available())
    {
      /* Enter menu mode when the user press a key */
      Serial.read();
      ShowMenu();
      menu_mode = 1;
    } else {
      /* Output current time every second */
      ShowCurrentTime();
      delay(1000);
    }
  }
}  
